/*****************************************************************************
 *
 *  confaudmp4mencoderdialog.h -- Input the MP4 audio codec parameters.
 *
 *  Copyright (C) 2010 Starbuck <starbuck@labdv.com>
 *  All rights reserved.
 *
 *  This file is part of the Gui4 module of the DVDx 4 application.
 *
 *  GNU Lesser General Public License Usage
 *  This file may be used under the terms of the GNU Lesser General
 *  Public License version 2.1 as published by the Free Software
 *  Foundation and appearing in the file LICENSE.LGPL included in the
 *  packaging of this file.  Please review the following information to
 *  ensure the GNU Lesser General Public License version 2.1 requirements
 *  will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 *  GNU General Public License Usage
 *  Alternatively, this file may be used under the terms of the GNU
 *  General Public License version 3.0 as published by the Free Software
 *  Foundation and appearing in the file LICENSE.GPL included in the
 *  packaging of this file.  Please review the following information to
 *  ensure the GNU General Public License version 3.0 requirements will be
 *  met: http://www.gnu.org/copyleft/gpl.html.
 *
 *****************************************************************************/

#ifndef CONFAUDMP4MENCODERDIALOG_H
#define CONFAUDMP4MENCODERDIALOG_H

#include "../video/video_struct.h"

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QComboBox>
#include <QtGui/QDialog>
#include <QtGui/QDialogButtonBox>
#include <QtGui/QGroupBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QRadioButton>
#include <QtGui/QSlider>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>


class ConfAudMp4MEncoderDialog : public QDialog
{
	Q_OBJECT

public:
	ConfAudMp4MEncoderDialog(encoder_audio_settings_t *settings, 
		audio_t *track,
		QWidget *parent = 0);

private slots:
	void accept();
	void bitrateSliderChanged(const int);
	void bitrateLineEditChanged(const QString &);
	void qualitySliderChanged(const int);
	void qualityLineEditChanged(const QString &);
	void toggleQualityMode(bool force = false, bool fvalue = true);
	void toggleCutOff();
	void cutoffSliderChanged(const int);
	void cutoffLineEditChanged(const QString &);
	void selectSampling(const int);
	void selectObject(const int);
	void selectPreset();
	void defaultPushButtonClicked();

private:
	void setToolTip();
	QString labelToolTip( const QString &defaults, const QString &details );
	QString editToolTip( const QString &name, const QString &range = tr("") );
	QString boxToolTip( const QString &name, const QString &defaults, const QString &range, const QString &details );

	void populateSampling();
	void populateObject();

	void updateControls();

	// Tabs
	QTabWidget *tabWidget;
	QDialogButtonBox *buttonBox;
	QVBoxLayout *verticalLayout;
	QWidget *mainTab;

	// Main tab
	QVBoxLayout *mainTabVLayout;
	QGroupBox *brqGroupBox;
	QVBoxLayout *brqVLayout;
	QHBoxLayout *brqHLayout;
	QSlider *bitrateSlider;			
	QLineEdit *bitrateLineEdit;
	QSlider *qualitySlider;			
	QLineEdit *qualityLineEdit;
	QPushButton *brqButton;

	QGroupBox *srateGroupBox;
	QHBoxLayout *srateHLayout;
	QComboBox *srateComboBox;
	QSpacerItem *srateHSpacer;

	QGroupBox *extraGroupBox;
	QVBoxLayout *extraVLayout;
	QHBoxLayout *cutoffHLayout;
	QCheckBox *cutoffCheckBox;
	QSlider *cutoffSlider;
	QLineEdit *cutoffLineEdit;
	QHBoxLayout *objectHLayout;
	QComboBox *objectComboBox;
	QHBoxLayout *tnsHLayout;
	QCheckBox *tnsCheckBox;

	QGroupBox *presetGroupBox;
	QVBoxLayout *presetVLayout;
	QRadioButton *presetNoneRadioButton;
	QRadioButton *presetVHQRadioButton;
	QRadioButton *presetHQRadioButton;
	QRadioButton *presetMQRadioButton;
	QRadioButton *presetDRRadioButton;
	QRadioButton *presetLRRadioButton;

	QHBoxLayout *defaultHLayout;
	QPushButton *defaultPushButton;
	QSpacerItem *defaultHSpacer;

	QSpacerItem *verticalSpacer;

	encoder_audio_settings_t *callerSettings;  // settings to modify
	encoder_audio_settings_t *encSettings;  // local copy
	audio_t *audioSource;  // for recommended stuff (rez, fps)

	QIntValidator *bitrateValidator;
	QIntValidator *qualityValidator;
	QIntValidator *cutoffValidator;

	bool isQualityMode;

	int bitrateMin, bitrateMax;
	int qualityMin, qualityMax;
	int cutoffMin,  cutoffMax;
};

#endif
