/*****************************************************************************
 *
 * confvidmpeg2mencoderdialog.cpp -- Input the lavc_mpeg2video codec parameters.
 *
 * Copyright (C) 2010 Starbuck <starbuck@labdv.com>
 *
 *****************************************************************************/

#include "../video/video_functions.h"

#include "confvidmpeg2mencoderdialog.h"
#include "userdefinedrezdialog.h"
#include "userdefinedfpsdialog.h"
#include "../encoder/encoder.h"
#include "macros.h"
#include <QtGui>

ConfVidMpeg2MEncoderDialog::ConfVidMpeg2MEncoderDialog(encoder_video_settings_t *settings, 
	video_t *track, QWidget *parent)
	: QDialog(parent),
	suspendMuxerSettingsUpdate(false),
	bitrateMin( 1600 ),
	bitrateMax( 9800 )
{
	callerSettings = settings;
	videoSource = track;

	// Local copy allows to cancel changes
	encSettings = new encoder_video_settings_t( *settings );

	// Ensure consistent MPEG-1 settings
	encSettings->mpeg2.resX = qMax( encSettings->mpeg2.resX, (int) widthMin );
	encSettings->mpeg2.resX = qMin( encSettings->mpeg2.resX, (int) widthMax );
	encSettings->mpeg2.resY = qMax( encSettings->mpeg2.resY, (int) heightMin );
	encSettings->mpeg2.resY = qMin( encSettings->mpeg2.resY, (int) heightMax );
	encSettings->mpeg2.bitrate = qMax( encSettings->mpeg2.bitrate, bitrateMin );
	encSettings->mpeg2.bitrate = qMin( encSettings->mpeg2.bitrate, bitrateMax );
	if (encSettings->mpeg2.mpeg_fps < 1 || encSettings->mpeg2.mpeg_fps > 8) encSettings->mpeg2.mpeg_fps = 4;

	setMinimumSize( 280, 360 );
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

	verticalLayout = new QVBoxLayout(this);

	// Global buttons
	buttonBox = new QDialogButtonBox(this);

	buttonBox->setOrientation(Qt::Horizontal);
	buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

	// Build all tabs
	tabWidget = new QTabWidget(this);
	tabWidget->setMovable(false);

	mainTab = new QWidget();
	advancedTab = new QWidget();
	pictureTab = new QWidget();
	muxerTab = new QWidget();

	tabWidget->addTab(mainTab, QString());
	tabWidget->addTab(advancedTab, QString());
	tabWidget->addTab(pictureTab, QString());
	tabWidget->addTab(muxerTab, QString());

	verticalLayout->addWidget(tabWidget);
	verticalLayout->addWidget(buttonBox);

	tabWidget->setCurrentIndex(0);

	// Main tab 
	mainTabVLayout = new QVBoxLayout(mainTab);
	mainTabVLayout->setSpacing(12);

	// - TV standard
	tvGroupBox = new QGroupBox(mainTab);
	tvVLayout = new QVBoxLayout(tvGroupBox);
	tvNtscCheckBox = new QCheckBox(tvGroupBox);
	tvPalCheckBox = new QCheckBox(tvGroupBox);
	
	tvVLayout->addWidget(tvNtscCheckBox);
	tvVLayout->addWidget(tvPalCheckBox);

	mainTabVLayout->addWidget(tvGroupBox);

	// - Rez and framerate
	frameGroupBox = new QGroupBox(mainTab);
	horizontalLayout = new QHBoxLayout(frameGroupBox);
	rezComboBox = new QComboBox(frameGroupBox);
	fpsComboBox = new QComboBox(frameGroupBox);

	horizontalLayout->addWidget(rezComboBox);
	horizontalLayout->addWidget(fpsComboBox);

	mainTabVLayout->addWidget(frameGroupBox);

	// - Bitrate
	bitrateGroupBox = new QGroupBox(mainTab);
	bitrateVLayout = new QVBoxLayout(bitrateGroupBox);
	bitrateHLayout = new QHBoxLayout();
	bitrateSlider = new QSlider(bitrateGroupBox);
	bitrateLineEdit = new QLineEdit(bitrateGroupBox);

	bitrateSlider->setOrientation(Qt::Horizontal);
	bitrateSlider->setInvertedAppearance(false);
	bitrateSlider->setTickPosition(QSlider::NoTicks);

	bitrateSlider->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	bitrateLineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	bitrateLineEdit->setMaxLength(6);
	bitrateLineEdit->setMaximumWidth(50);

	bitrateHLayout->addWidget(bitrateSlider);
	bitrateHLayout->addWidget(bitrateLineEdit);

	bitrateVLayout->addLayout(bitrateHLayout);

	mainTabVLayout->addWidget(bitrateGroupBox);

	// - Stream format
	formatGroupBox = new QGroupBox(mainTab);
	formatHLayout = new QHBoxLayout(formatGroupBox);
	formatComboBox = new QComboBox(formatGroupBox);
	formatSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	formatHLayout->addWidget(formatComboBox);
	formatHLayout->addItem(formatSpacer);

	mainTabVLayout->addWidget(formatGroupBox);

	// - Preset
	presetGroupBox = new QGroupBox(mainTab);
	presetVLayout = new QVBoxLayout(presetGroupBox);
	presetDvdCheckBox = new QCheckBox(presetGroupBox);
	presetDvdLpCheckBox = new QCheckBox(presetGroupBox);
	presetSvcdCheckBox = new QCheckBox(presetGroupBox);
	
	presetVLayout->addWidget(presetDvdCheckBox);
	presetVLayout->addWidget(presetDvdLpCheckBox);
	presetVLayout->addWidget(presetSvcdCheckBox);

	mainTabVLayout->addWidget(presetGroupBox);

	// - Spacer
	verticalSpacer = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);

	mainTabVLayout->addItem(verticalSpacer);

	// Advanced tab
	advancedTabVLayout = new QVBoxLayout(advancedTab);
	advancedTabVLayout->setSpacing(12);

	// - Bitrate allocation
	br_allocGroupBox = new QGroupBox(advancedTab);
	br_allocVLayout = new QVBoxLayout(br_allocGroupBox);
	br_allocComboBox = new QComboBox(br_allocGroupBox);
	vbrGLayout = new QGridLayout();
	
	br_allocVLayout->addWidget(br_allocComboBox);
	br_allocVLayout->addLayout(vbrGLayout);

	advancedTabVLayout->addWidget(br_allocGroupBox);

	// -- VBR nominal
	vbr_nomLabel = new QLabel(br_allocGroupBox);
	vbr_nomSlider = new QSlider(br_allocGroupBox);
	vbr_nomLineEdit = new QLineEdit(br_allocGroupBox);

	vbr_nomSlider->setOrientation(Qt::Horizontal);
	vbr_nomSlider->setInvertedAppearance(false);
	vbr_nomSlider->setTickPosition(QSlider::NoTicks);

	vbr_nomLabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	vbr_nomSlider->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
	vbr_nomLineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	vbr_nomLineEdit->setMaxLength(4);
	vbr_nomLineEdit->setMaximumWidth(40);

	vbrGLayout->addWidget(vbr_nomLabel,		0, 0);
	vbrGLayout->addWidget(vbr_nomSlider,	0, 1);
	vbrGLayout->addWidget(vbr_nomLineEdit,	0, 2);

	// -- VBR min
	vbr_minLabel = new QLabel(br_allocGroupBox);
	vbr_minSlider = new QSlider(br_allocGroupBox);
	vbr_minLineEdit = new QLineEdit(br_allocGroupBox);

	vbr_minSlider->setOrientation(Qt::Horizontal);
	vbr_minSlider->setInvertedAppearance(false);
	vbr_minSlider->setTickPosition(QSlider::NoTicks);

	vbr_minLabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	vbr_minSlider->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
	vbr_minLineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	vbr_minLineEdit->setMaxLength(4);
	vbr_minLineEdit->setMaximumWidth(40);

	vbrGLayout->addWidget(vbr_minLabel,		1, 0);
	vbrGLayout->addWidget(vbr_minSlider,	1, 1);
	vbrGLayout->addWidget(vbr_minLineEdit,	1, 2);

	// -- VBR max
	vbr_maxLabel = new QLabel(br_allocGroupBox);
	vbr_maxSlider = new QSlider(br_allocGroupBox);
	vbr_maxLineEdit = new QLineEdit(br_allocGroupBox);

	vbr_maxSlider->setOrientation(Qt::Horizontal);
	vbr_maxSlider->setInvertedAppearance(false);
	vbr_maxSlider->setTickPosition(QSlider::NoTicks);

	vbr_minLabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	vbr_maxSlider->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
	vbr_maxLineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	vbr_maxLineEdit->setMaxLength(4);
	vbr_maxLineEdit->setMaximumWidth(40);

	vbrGLayout->addWidget(vbr_maxLabel,		2, 0);
	vbrGLayout->addWidget(vbr_maxSlider,	2, 1);
	vbrGLayout->addWidget(vbr_maxLineEdit,	2, 2);

	// - GOP
	gopGroupBox = new QGroupBox(advancedTab);
	gopVLayout = new QVBoxLayout(gopGroupBox);

	gop_sizeHLayout = new QHBoxLayout();
	gop_sizeLineEdit = new QLineEdit(gopGroupBox);
	gop_sizeLineEdit->setMaxLength(3);
	gop_sizeLineEdit->setMaximumSize(QSize(50, 40));
	gop_sizeLabel = new QLabel(gopGroupBox);
	gop_sizeHSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	gop_sizeHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	gop_sizeHLayout->addWidget(gop_sizeLineEdit);
	gop_sizeHLayout->addWidget(gop_sizeLabel);
	gop_sizeHLayout->addItem(gop_sizeHSpacer);

	closed_gopCheckBox = new QCheckBox(gopGroupBox);

	gopVLayout->addLayout(gop_sizeHLayout);
	gopVLayout->addWidget(closed_gopCheckBox);

	advancedTabVLayout->addWidget(gopGroupBox);

	// - Spacer
	advancedTabSpacer = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);

	advancedTabVLayout->addItem(advancedTabSpacer);

	// Picture tab
	pictureTabVLayout = new QVBoxLayout(pictureTab);
	pictureTabVLayout->setSpacing(12);

	// - Aspect ratio
	aspectGroupBox = new QGroupBox(pictureTab);
	aspectHLayout = new QHBoxLayout(aspectGroupBox);
	aspectComboBox = new QComboBox(aspectGroupBox);
	aspectSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	aspectHLayout->addWidget(aspectComboBox);
	aspectHLayout->addItem(aspectSpacer);

	pictureTabVLayout->addWidget(aspectGroupBox);

	// - Pan Scan
	panscanGroupBox = new QGroupBox(pictureTab);
	panscanHLayout = new QHBoxLayout(panscanGroupBox);
	panscanCheckBox = new QCheckBox(panscanGroupBox);
	pswidthLineEdit = new QLineEdit(panscanGroupBox);
	pswidthLineEdit->setMaximumSize(QSize(50, 40));
	psheightLineEdit = new QLineEdit(panscanGroupBox);
	psheightLineEdit->setMaximumSize(QSize(50, 40));
	panscanLabel = new QLabel( "x", panscanGroupBox );
	panscanSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	panscanHLayout->addWidget(panscanCheckBox);
	panscanHLayout->addWidget(pswidthLineEdit);
	panscanHLayout->addWidget(panscanLabel);
	panscanHLayout->addWidget(psheightLineEdit);
	panscanHLayout->addItem(panscanSpacer);

	pictureTabVLayout->addWidget(panscanGroupBox);

	// - Telecine
	telecineGroupBox = new QGroupBox(pictureTab);
	telecineVLayout = new QVBoxLayout(telecineGroupBox);
	telecineNoneRadioButton = new QRadioButton(telecineGroupBox);
	telecine32PulldownRadioButton = new QRadioButton(telecineGroupBox);
	telecineFilm2PalRadioButton = new QRadioButton(telecineGroupBox);
	telecineDGPulldownRadioButton = new QRadioButton(telecineGroupBox);

	dgPulldownHLayout = new QHBoxLayout();
	dgSrcComboBox = new QComboBox(telecineGroupBox);
	dgDestComboBox = new QComboBox(telecineGroupBox);
	dgPulldownSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	dgPulldownHLayout->addWidget(telecineDGPulldownRadioButton);
	dgPulldownHLayout->addWidget(dgSrcComboBox);
	dgPulldownHLayout->addWidget(dgDestComboBox);
	dgPulldownHLayout->addItem(dgPulldownSpacer);
	
	telecineVLayout->addWidget(telecineNoneRadioButton);
	telecineVLayout->addWidget(telecine32PulldownRadioButton);
	telecineVLayout->addWidget(telecineFilm2PalRadioButton);
	telecineVLayout->addLayout(dgPulldownHLayout);

	pictureTabVLayout->addWidget(telecineGroupBox);

	// - Spacer
	pictureVSpacer = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);

	pictureTabVLayout->addItem(pictureVSpacer);

	// - Default picture settings
	pictureDefaultHLayout = new QHBoxLayout();
	pictureDefaultPushButton = new QPushButton(pictureTab);
	pictureDefaultHSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	pictureDefaultHLayout->addWidget(pictureDefaultPushButton);
	pictureDefaultHLayout->addItem(pictureDefaultHSpacer);

	pictureTabVLayout->addLayout(pictureDefaultHLayout);

	// Muxer tab
	muxerTabVLayout = new QVBoxLayout(muxerTab);
	muxerTabVLayout->setSpacing(12);

	// - MPEG A/V Muxer settings
	muxerGroupBox = new QGroupBox(muxerTab);
	muxerVLayout = new QVBoxLayout(muxerGroupBox);

	muxerTabVLayout->addWidget(muxerGroupBox);

	// -- Pack size
	pack_sizeHLayout = new QHBoxLayout();
	pack_sizeLineEdit = new QLineEdit(muxerGroupBox);
	pack_sizeLineEdit->setMaximumSize(QSize(50, 40));
	pack_sizeLabel = new QLabel(muxerGroupBox);
	pack_sizeSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	pack_sizeHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	pack_sizeHLayout->addWidget(pack_sizeLineEdit);
	pack_sizeHLayout->addWidget(pack_sizeLabel);
	pack_sizeHLayout->addItem(pack_sizeSpacer);

	muxerVLayout->addLayout(pack_sizeHLayout);

	// -- Muxer rate
	muxrateHLayout = new QHBoxLayout();
	muxrateLineEdit = new QLineEdit(muxerGroupBox);
	muxrateLineEdit->setMaximumSize(QSize(50, 40));
	muxrateLabel = new QLabel(muxerGroupBox);
	muxrateSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	muxrateHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	muxrateHLayout->addWidget(muxrateLineEdit);
	muxrateHLayout->addWidget(muxrateLabel);
	muxrateHLayout->addItem(muxrateSpacer);

	muxerVLayout->addLayout(muxrateHLayout);

	// -- Timestamps on all frames if possible
	tsafHLayout = new QHBoxLayout();
	tsafCheckBox = new QCheckBox(muxerGroupBox);
	tsafSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	tsafHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	tsafHLayout->addWidget(tsafCheckBox);
	tsafHLayout->addItem(tsafSpacer);

	muxerVLayout->addLayout(tsafHLayout);

	// -- Better interleaving
	interleaving2HLayout = new QHBoxLayout();
	interleaving2CheckBox = new QCheckBox(muxerGroupBox);
	interleaving2Spacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	interleaving2HLayout->setSizeConstraint(QLayout::SetMaximumSize);
	interleaving2HLayout->addWidget(interleaving2CheckBox);
	interleaving2HLayout->addItem(interleaving2Spacer);

	muxerVLayout->addLayout(interleaving2HLayout);

	// -- Video delay
	vdelayHLayout = new QHBoxLayout();
	vdelayLineEdit = new QLineEdit(muxerGroupBox);
	vdelayLineEdit->setMaximumSize(QSize(50, 40));
	vdelayLabel = new QLabel(muxerGroupBox);
	vdelaySpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	vdelayHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	vdelayHLayout->addWidget(vdelayLineEdit);
	vdelayHLayout->addWidget(vdelayLabel);
	vdelayHLayout->addItem(vdelaySpacer);

	muxerVLayout->addLayout(vdelayHLayout);

	// -- Audio delay
	adelayHLayout = new QHBoxLayout();
	adelayLineEdit = new QLineEdit(muxerGroupBox);
	adelayLineEdit->setMaximumSize(QSize(50, 40));
	adelayLabel = new QLabel(muxerGroupBox);
	adelaySpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	adelayHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	adelayHLayout->addWidget(adelayLineEdit);
	adelayHLayout->addWidget(adelayLabel);
	adelayHLayout->addItem(adelaySpacer);

	muxerVLayout->addLayout(adelayHLayout);

	// -- Drop anticipated audio
	dropHLayout = new QHBoxLayout();
	dropCheckBox = new QCheckBox(muxerGroupBox);
	dropSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	dropHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	dropHLayout->addWidget(dropCheckBox);
	dropHLayout->addItem(dropSpacer);

	muxerVLayout->addLayout(dropHLayout);

	// -- Video buffer size
	vbuf_sizeHLayout = new QHBoxLayout();
	vbuf_sizeLineEdit = new QLineEdit(muxerGroupBox);
	vbuf_sizeLineEdit->setMaximumSize(QSize(50, 40));
	vbuf_sizeLabel = new QLabel(muxerGroupBox);
	vbuf_sizeSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	vbuf_sizeHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	vbuf_sizeHLayout->addWidget(vbuf_sizeLineEdit);
	vbuf_sizeHLayout->addWidget(vbuf_sizeLabel);
	vbuf_sizeHLayout->addItem(vbuf_sizeSpacer);

	muxerVLayout->addLayout(vbuf_sizeHLayout);

	// -- Audio buffer size
	abuf_sizeHLayout = new QHBoxLayout();
	abuf_sizeLineEdit = new QLineEdit(muxerGroupBox);
	abuf_sizeLineEdit->setMaximumSize(QSize(50, 40));
	abuf_sizeLabel = new QLabel(muxerGroupBox);
	abuf_sizeSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	abuf_sizeHLayout->setSizeConstraint(QLayout::SetMaximumSize);
	abuf_sizeHLayout->addWidget(abuf_sizeLineEdit);
	abuf_sizeHLayout->addWidget(abuf_sizeLabel);
	abuf_sizeHLayout->addItem(abuf_sizeSpacer);

	muxerVLayout->addLayout(abuf_sizeHLayout);

	// - Muxer standard compliance
	muxerstdGroupBox = new QGroupBox();
	muxerstdVLayout = new QVBoxLayout(muxerstdGroupBox);
	muxerstdDvdFlagBox = new FlagBox(muxerstdGroupBox);
	muxerstSvcdFlagBox = new FlagBox(muxerstdGroupBox);

	muxerstdDvdFlagBox->setGreenLightStyle();
	muxerstSvcdFlagBox->setGreenLightStyle();

	muxerstdVLayout->addWidget(muxerstdDvdFlagBox);
	muxerstdVLayout->addWidget(muxerstSvcdFlagBox);

	muxerVLayout->addWidget(muxerstdGroupBox);

	// - Spacer
	muxerVSpacer = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);

	muxerTabVLayout->addItem(muxerVSpacer);

	// - Default muxer settings
	muxerDefaultHLayout = new QHBoxLayout();
	muxerDefaultPushButton = new QPushButton(muxerTab);
	muxerDefaultHSpacer = new QSpacerItem(40, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

	muxerDefaultHLayout->addWidget(muxerDefaultPushButton);
	muxerDefaultHLayout->addItem(muxerDefaultHSpacer);

	muxerTabVLayout->addLayout(muxerDefaultHLayout);

	// Captions
	setWindowTitle( tr("MPEG-2 encoder settings") );

	tabWidget->setTabText(tabWidget->indexOf(mainTab), tr("General") );
	tabWidget->setTabText(tabWidget->indexOf(advancedTab), tr("Advanced") );
	tabWidget->setTabText(tabWidget->indexOf(pictureTab), tr("Picture") );
	tabWidget->setTabText(tabWidget->indexOf(muxerTab), tr("Muxer") );

	br_allocGroupBox->setTitle( tr(" Bitrate allocation ") );
	vbr_nomLabel->setText( tr("- Nominal VBR") );
	vbr_minLabel->setText( tr("- Minimum VBR") );
	vbr_maxLabel->setText(   tr("- Maximum VBR") );
	gopGroupBox->setTitle( tr("  Group Of Pictures ") );
	gop_sizeLabel->setText( tr("GOP size") );
	closed_gopCheckBox->setText( tr("Closed GOP") );

	frameGroupBox->setTitle( tr(" Picture size - Frame rate ") );
	bitrateGroupBox->setTitle( tr(" Bitrate ( kbps ) ") );
	tvGroupBox->setTitle( tr(" TV standard ") );
	tvNtscCheckBox->setText( tr("NTSC") );
	tvPalCheckBox->setText( tr("PAL") );
	formatGroupBox->setTitle( tr(" MPEG stream format ") );
	presetGroupBox->setTitle( tr(" Preset discs ") );
	presetDvdCheckBox->setText( tr("DVD") );
	presetDvdLpCheckBox->setText( tr("DVD-LP") );
	presetSvcdCheckBox->setText( tr("SVCD") );

	muxerGroupBox->setTitle( tr(" MPEG audio-video muxer ") );

	pack_sizeLabel->setText( tr("MPEG muxer packet size") );
	muxrateLabel->setText( tr("MPEG muxer rate") );
	vdelayLabel->setText( tr("Video delay") );
	adelayLabel->setText( tr("Audio delay") );
	vbuf_sizeLabel->setText( tr("Video buffer size") );
	abuf_sizeLabel->setText( tr("Audio buffer size") );

	tsafCheckBox->setText( tr("Timestamps on all frames") );
	interleaving2CheckBox->setText( tr("Better algorithm to interleave audio and video packets") );
	dropCheckBox->setText( tr("Drop the part of audio anticipated with video delay") );

	muxerstdGroupBox->setTitle( tr(" Muxer standard compliance notification ") );
	muxerstdDvdFlagBox->setText( tr("DVD") );
	muxerstSvcdFlagBox->setText( tr("SVCD") );

	muxerDefaultPushButton->setText( tr("Default muxer settings") );

	aspectGroupBox->setTitle( tr(" Display aspect ratio ") );

	panscanGroupBox->setTitle( tr(" Pan-Scan ") );
	panscanCheckBox->setText( tr("Activated") );

	telecineGroupBox->setTitle( tr(" Telecine ") );
	telecineNoneRadioButton->setText( tr("None") );
	telecine32PulldownRadioButton->setText( tr("Inverse Telecine 3:2 Pulldown") );
	telecineFilm2PalRadioButton->setText( tr("Film to Pal / NTSC to PAL") );
	telecineDGPulldownRadioButton->setText( tr("Arbitrary Pulldown") );

	pictureDefaultPushButton->setText( tr("Default picture settings") );

	// Tool tips and What's this
	setToolTip();
	setWhatsThis();

	setAttribute(Qt::WA_AlwaysShowToolTips);

	bitrateSlider->setRange( bitrateMin, bitrateMax );
	vbr_nomSlider->setRange( bitrateMin, bitrateMax );
	vbr_minSlider->setRange( bitrateMin, bitrateMax );
	vbr_maxSlider->setRange( bitrateMin, bitrateMax );

	bitrateValidator = new QIntValidator( bitrateMin, bitrateMax, this );
	vbr_nomValidator = new QIntValidator( bitrateMin, encSettings->mpeg2.bitrate, this );
	vbr_minValidator = new QIntValidator( bitrateMin, encSettings->mpeg2.bitrate, this );
	vbr_maxValidator = new QIntValidator( encSettings->mpeg2.bitrate, bitrateMax, this );

	bitrateLineEdit->setValidator( bitrateValidator );
	vbr_nomLineEdit->setValidator( vbr_nomValidator );
	vbr_minLineEdit->setValidator( vbr_minValidator );
	vbr_maxLineEdit->setValidator( vbr_maxValidator );

	populateFps(); // may alter tvButtons
	populateRez();
	populateFormat();
	populateBrAllocation();
	populateAspect();
	populateTeleSrc();
	populateTeleDest();

	// Signals which don't change settings
	connect(bitrateLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(bitrateLineEditChanged(QString)));
	connect(bitrateSlider, SIGNAL(valueChanged(int)), 
		this, SLOT(bitrateSliderChanged(int)));

	connect(vbr_nomLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(vbr_nomLineEditChanged(QString)));
	connect(vbr_nomSlider, SIGNAL(valueChanged(int)), 
		this, SLOT(vbr_nomSliderChanged(int)));

	connect(vbr_minLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(vbr_minLineEditChanged(QString)));
	connect(vbr_minSlider, SIGNAL(valueChanged(int)), 
		this, SLOT(vbr_minSliderChanged(int)));
	connect(vbr_minSlider, SIGNAL(sliderReleased()), 
		this, SLOT(vbr_minSliderReleased()));

	connect(vbr_maxLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(vbr_maxLineEditChanged(QString)));
	connect(vbr_maxSlider, SIGNAL(valueChanged(int)), 
		this, SLOT(vbr_maxSliderChanged(int)));
	connect(vbr_maxSlider, SIGNAL(sliderReleased()), 
		this, SLOT(vbr_maxSliderReleased()));

	// Update controls from current settings before connecting signals
	updateControls();

	// Now connect other signals (which controls are up-to-date)
	connect(rezComboBox, SIGNAL(activated(int)), 
		this, SLOT(selectRez(int)));

	connect(fpsComboBox, SIGNAL(activated(int)), 
		this, SLOT(selectFps(int)));

	connect(formatComboBox, SIGNAL(activated(int)), 
		this, SLOT(selectFormat(int)));

	connect(tvNtscCheckBox, SIGNAL(clicked()), 
		this, SLOT(selectNtscTv()));
	connect(tvPalCheckBox, SIGNAL(clicked()), 
		this, SLOT(selectPalTv()));

	connect(presetDvdCheckBox, SIGNAL(clicked()), 
		this, SLOT(selectPresetDvd()));
	connect(presetDvdLpCheckBox, SIGNAL(clicked()), 
		this, SLOT(selectPresetDvdLp()));
	connect(presetSvcdCheckBox, SIGNAL(clicked()), 
		this, SLOT(selectPresetSvcd()));

	connect(br_allocComboBox, SIGNAL(activated(int)), 
		this, SLOT(selectBrAllocation(int)));

	connect(gop_sizeLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(gop_sizeLineEditChanged(QString)));
	connect(closed_gopCheckBox, SIGNAL(clicked()), 
		this, SLOT(closed_gopCheckBoxClicked()));

	connect(pack_sizeLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(muxrateLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(vdelayLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(adelayLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(vbuf_sizeLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(abuf_sizeLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updateMpegMuxerSettings()));

	connect(tsafCheckBox, SIGNAL(toggled(bool)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(interleaving2CheckBox, SIGNAL(toggled(bool)), 
		this, SLOT(updateMpegMuxerSettings()));
	connect(dropCheckBox, SIGNAL(toggled(bool)), 
		this, SLOT(updateMpegMuxerSettings()));

	connect(muxerDefaultPushButton, SIGNAL(clicked()), 
		this, SLOT(muxerDefaultPushButtonClicked()));

	connect(aspectComboBox, SIGNAL(activated(int)), 
		this, SLOT(selectAspect(int)));

	connect(panscanCheckBox, SIGNAL(toggled(bool)), 
		this, SLOT(updatePanScanSettings()));
	connect(pswidthLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updatePanScanSettings()));
	connect(psheightLineEdit, SIGNAL(textChanged(QString)), 
		this, SLOT(updatePanScanSettings()));

	connect(telecineNoneRadioButton, SIGNAL(clicked()), 
		this, SLOT(selectTelecine()));
	connect(telecine32PulldownRadioButton, SIGNAL(clicked()), 
		this, SLOT(selectTelecine()));
	connect(telecineFilm2PalRadioButton, SIGNAL(clicked()), 
		this, SLOT(selectTelecine()));
	connect(telecineDGPulldownRadioButton, SIGNAL(clicked()), 
		this, SLOT(selectTelecine()));

	connect(dgSrcComboBox, SIGNAL(currentIndexChanged(int)), 
		this, SLOT(selectDgSrc(int)));
	connect(dgDestComboBox, SIGNAL(currentIndexChanged(int)), 
		this, SLOT(selectDgDest(int)));

	connect(pictureDefaultPushButton, SIGNAL(clicked()), 
		this, SLOT(pictureDefaultPushButtonClicked()));

	// Global buttons
	connect(buttonBox, SIGNAL(rejected()), this, SLOT(close()));
	connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
}

void ConfVidMpeg2MEncoderDialog::accept()
{
	*callerSettings = *encSettings;

	QDialog::accept();
}

void ConfVidMpeg2MEncoderDialog::updateControls()
{
	int i;

	// State flags
	bool ntsc, pal, dvd, dvdlp, svcd;

	suspendMuxerSettingsUpdate = true;

	ntsc = ( encSettings->mpeg2.resX == 352 || encSettings->mpeg2.resX == 480 
				|| encSettings->mpeg2.resX == 704 || encSettings->mpeg2.resX == 720 )
		&& encSettings->mpeg2.resY == 480
		&& ( encSettings->mpeg2.mpeg_fps == 1 || encSettings->mpeg2.mpeg_fps == 4 || encSettings->mpeg2.mpeg_fps == 7 );

	pal = ( encSettings->mpeg2.resX == 352 || encSettings->mpeg2.resX == 480 
				|| encSettings->mpeg2.resX == 704 || encSettings->mpeg2.resX == 720 )
		&& encSettings->mpeg2.resY == 576
		&& ( encSettings->mpeg2.mpeg_fps == 3 || encSettings->mpeg2.mpeg_fps == 6 );

	dvd = ( encSettings->mpeg2.resX == 352 || encSettings->mpeg2.resX == 704 || encSettings->mpeg2.resX == 720 )
		&& ( ntsc || pal )
		&& encSettings->mpeg2.stream == "dvd"
		&& encSettings->mpeg2.bitrate >= 5000
		&& encSettings->mpeg2.bitrate <= 10000;

	dvdlp = ( encSettings->mpeg2.resX == 352 || encSettings->mpeg2.resX == 704 || encSettings->mpeg2.resX == 720 )
		&& ( ntsc || pal )
		&& encSettings->mpeg2.stream == "dvd"
		&& encSettings->mpeg2.bitrate < 5000;

	svcd = (( ntsc && encSettings->mpeg2.resX == 480 && encSettings->mpeg2.resY == 480 )
		|| ( pal && encSettings->mpeg2.resX == 480 && encSettings->mpeg2.resY == 576 ))
		&& encSettings->mpeg2.stream == "xsvcd"
		&& encSettings->mpeg2.bitrate >= 2400
		&& encSettings->mpeg2.bitrate <= 2600;

	// Update GUI
	bitrateSlider->setValue( encSettings->mpeg2.bitrate );
	bitrateLineEdit->setText( QString::number(encSettings->mpeg2.bitrate) );

	vbr_nomSlider->setValue( encSettings->mpeg2.bitrate );
	vbr_nomLineEdit->setText( QString::number(encSettings->mpeg2.bitrate) );

	vbr_minSlider->setValue( encSettings->mpeg2.minvbr );
	vbr_minLineEdit->setText( QString::number(encSettings->mpeg2.minvbr) );

	vbr_maxSlider->setValue( encSettings->mpeg2.maxvbr );
	vbr_maxLineEdit->setText( QString::number(encSettings->mpeg2.maxvbr) );

	vbr_minValidator->setRange( bitrateMin, encSettings->mpeg2.bitrate );
	vbr_maxValidator->setRange( encSettings->mpeg2.bitrate, bitrateMax );

	tvNtscCheckBox->setChecked( ntsc );
	tvPalCheckBox->setChecked( pal );

	presetDvdCheckBox->setChecked( dvd );
	presetDvdLpCheckBox->setChecked( dvdlp );
	presetSvcdCheckBox->setChecked( svcd );

	gop_sizeLineEdit->setText( QString::number(encSettings->mpeg2.gop_size) );
	closed_gopCheckBox->setChecked( encSettings->mpeg2.closed_gop );

	if ( encSettings->mpeg2.cbr )
	{
		br_allocComboBox->setCurrentIndex( 0 ); // CBR

		vbr_minLabel->setEnabled(false);
		vbr_minSlider->setEnabled(false);
		vbr_minLineEdit->setEnabled(false);
		vbr_maxLabel->setEnabled(false);
		vbr_maxSlider->setEnabled(false);
		vbr_maxLineEdit->setEnabled(false);
	}
	else
	{
		br_allocComboBox->setCurrentIndex( 1 ); // VBR
		
		vbr_minLabel->setEnabled(true);
		vbr_minSlider->setEnabled(true);
		vbr_minLineEdit->setEnabled(true);
		vbr_maxLabel->setEnabled(true);
		vbr_maxSlider->setEnabled(true);
		vbr_maxLineEdit->setEnabled(true);		
	}

	i = rezComboBox->findData( QSize(encSettings->mpeg2.resX, encSettings->mpeg2.resY) );

	if ( i == -1 )
	{
		rezComboBox->setItemData( 1, QSize(encSettings->mpeg2.resX, encSettings->mpeg2.resY) );
		rezComboBox->setItemText( 1, tr("%1x%2 (user defined)")
										.arg(encSettings->mpeg2.resX)
										.arg(encSettings->mpeg2.resY) );
		rezComboBox->setCurrentIndex( 1 );
	}
	else
	{
		rezComboBox->setCurrentIndex( i );
	}
	
	i = fpsComboBox->findData( encSettings->mpeg2.mpeg_fps );

	if ( i == -1 ) // should not happened
	{
		fpsComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.mpeg_fps = fpsComboBox->itemData( 0 ).toInt();
	}
	else
	{
		fpsComboBox->setCurrentIndex( i );
	}

	i = formatComboBox->findData( encSettings->mpeg2.stream );

	if ( i == -1 ) // should not happened
	{
		formatComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.stream = formatComboBox->itemData( 0 ).toInt();
	}
	else
	{
		formatComboBox->setCurrentIndex( i );
	}

	pack_sizeLineEdit->setText( QString::number(encSettings->mpeg2.pack_size) );
	muxrateLineEdit->setText( QString::number(encSettings->mpeg2.muxrate) );
	vdelayLineEdit->setText( QString::number(encSettings->mpeg2.vdelay) );
	adelayLineEdit->setText( QString::number(encSettings->mpeg2.adelay) );
	vbuf_sizeLineEdit->setText( QString::number(encSettings->mpeg2.vbuf_size) );
	abuf_sizeLineEdit->setText( QString::number(encSettings->mpeg2.abuf_size) );

	tsafCheckBox->setChecked( encSettings->mpeg2.tsaf );
	interleaving2CheckBox->setChecked( encSettings->mpeg2.interleaving2 );
	dropCheckBox->setChecked( encSettings->mpeg2.drop );

	i = aspectComboBox->findData( encSettings->mpeg2.mpeg_aspect );

	if ( i == -1 ) // should not happened
	{
		aspectComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.mpeg_aspect = aspectComboBox->itemData( 0 ).toInt();
	}
	else
	{
		aspectComboBox->setCurrentIndex( i );
	}

	panscanCheckBox->setChecked( encSettings->mpeg2.vpswidth || encSettings->mpeg2.vpsheight );
	pswidthLineEdit->setEnabled( encSettings->mpeg2.vpswidth || encSettings->mpeg2.vpsheight );
	psheightLineEdit->setEnabled( encSettings->mpeg2.vpswidth || encSettings->mpeg2.vpsheight );
	pswidthLineEdit->setText( QString::number(encSettings->mpeg2.vpswidth) );
	psheightLineEdit->setText( QString::number(encSettings->mpeg2.vpsheight) );

	telecine32PulldownRadioButton->setChecked( encSettings->mpeg2.telecine );
	telecineFilm2PalRadioButton->setChecked( encSettings->mpeg2.film2pal );
	telecineDGPulldownRadioButton->setChecked( encSettings->mpeg2.tele_src || encSettings->mpeg2.tele_dest );
	telecineNoneRadioButton->setChecked( !(encSettings->mpeg2.telecine || encSettings->mpeg2.film2pal
											|| encSettings->mpeg2.tele_src || encSettings->mpeg2.tele_dest) );

	muxerstdDvdFlagBox->setChecked( encSettings->mpeg2.pack_size == 2048 );
	muxerstSvcdFlagBox->setChecked( encSettings->mpeg2.pack_size == 2324 );

	suspendMuxerSettingsUpdate = false;
}

void ConfVidMpeg2MEncoderDialog::updateMpegMuxerSettings()
{
	if (suspendMuxerSettingsUpdate) return;

	encSettings->mpeg2.pack_size = pack_sizeLineEdit->text().toInt();		
	encSettings->mpeg2.muxrate = muxrateLineEdit->text().toInt();
	encSettings->mpeg2.tsaf = tsafCheckBox->isChecked();
	encSettings->mpeg2.interleaving2 = interleaving2CheckBox->isChecked();
	encSettings->mpeg2.vdelay = vdelayLineEdit->text().toInt();
	encSettings->mpeg2.adelay = adelayLineEdit->text().toInt();
	encSettings->mpeg2.drop = dropCheckBox->isChecked();
	encSettings->mpeg2.vbuf_size = vbuf_sizeLineEdit->text().toInt();
	encSettings->mpeg2.abuf_size = abuf_sizeLineEdit->text().toInt();

	muxerstdDvdFlagBox->setChecked( encSettings->mpeg2.pack_size == 2048 );
	muxerstSvcdFlagBox->setChecked( encSettings->mpeg2.pack_size == 2324 );
}

void ConfVidMpeg2MEncoderDialog::updatePanScanSettings()
{
	if ( !panscanCheckBox->isChecked() )
	{
		encSettings->mpeg2.vpswidth = 0;		
		encSettings->mpeg2.vpsheight = 0;

		pswidthLineEdit->setText( QString::number(encSettings->mpeg2.vpswidth) );
		psheightLineEdit->setText( QString::number(encSettings->mpeg2.vpsheight) );

		pswidthLineEdit->setEnabled(false);
		psheightLineEdit->setEnabled(false);
	}
	else
	{
		encSettings->mpeg2.vpswidth = pswidthLineEdit->text().toInt();		
		encSettings->mpeg2.vpsheight = psheightLineEdit->text().toInt();

		if ( encSettings->mpeg2.vpswidth == 0 ) encSettings->mpeg2.vpswidth = videoSource->resX;
		if ( encSettings->mpeg2.vpsheight == 0 ) encSettings->mpeg2.vpsheight = videoSource->resY;

		pswidthLineEdit->setText( QString::number(encSettings->mpeg2.vpswidth) );
		psheightLineEdit->setText( QString::number(encSettings->mpeg2.vpsheight) );

		pswidthLineEdit->setEnabled(true);
		psheightLineEdit->setEnabled(true);
	}
}

void ConfVidMpeg2MEncoderDialog::enterUserDefinedSize()
{
	QSize size(encSettings->user_defined.mpeg2.resX, encSettings->user_defined.mpeg2.resY);

	QSize min = QSize(widthMin, heightMin);
	QSize max = QSize(widthMax, heightMax);

	UserDefinedRezDialog *dlg = new UserDefinedRezDialog(&size, min, max, this);
	int res = dlg->exec();		
	Q_CHECK_PTR(dlg);
	delete(dlg);

	if ( res == QDialog::Rejected ) 
	{
		return;
	}

	encSettings->user_defined.mpeg2.resX = size.width();
	encSettings->user_defined.mpeg2.resY = size.height();

	// Update combo box
	rezComboBox->setItemText( 1, tr("%1x%2 (user defined)")
		.arg(size.width())
		.arg(size.height()));
	rezComboBox->setItemData( 1, size);

	// Select user defined rezolution
	rezComboBox->setCurrentIndex( 1 );

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::populateFormat()
{
	// Build a list of MPEG stream format values and its bi-univoq labels
	QList<QString> formatValues;
	formatValues << "dvd"
		<< "mpeg2"
		<< "xsvcd"
		<< "pes1"
		<< "pes2";

	QStringList formatLabels;
	formatLabels << "DVD"
		<< "MPEG-2"
		<< "Super-VideoCD"
		<< "PES1"
		<< "PES2";

	// Item #2... (push every common fps) 
	while (!formatValues.isEmpty() )
	{
		QString value;
		QString label;
		
		value = formatValues.takeFirst();
		label = formatLabels.takeFirst();
	
		formatComboBox->addItem( label, value );
	}

	// Select current fps
	int index = formatComboBox->findData( encSettings->mpeg2.stream );

	if ( index == -1 ) // should not happened
	{
		formatComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.stream = formatComboBox->itemData( 0 ).toInt();
	}
	else
	{
		formatComboBox->setCurrentIndex( index );
	}
}

void ConfVidMpeg2MEncoderDialog::populateFps()
{
	int mpeg_fps(0);

	fpsComboBox->clear();

	// Item #0 (first must be the recommended framerate)
	if ( tvNtscCheckBox->isChecked() )
	{
			mpeg_fps = (videoSource->fps != 23.976)? 4 : 1;
	}
	else if ( tvPalCheckBox->isChecked() )
	{
		mpeg_fps = 3;
	}
	else
	{
		mpeg_fps = Video::intDvdFps( videoSource->fps );
	}

	if ( mpeg_fps == 0 ) // forbidden
	{
		mpeg_fps = 2;	// 24 FPS
	}

	encSettings->recommendation.mpeg2.mpeg_fps = mpeg_fps;

	fpsComboBox->addItem( tr("%1 FPS (recommended)")
		.arg( Video::strDvdFps(mpeg_fps) ), mpeg_fps);
	
	// Build a list of MPEG authorized fps 
	QList<int> commonFps;
	commonFps << 1	// 23.976
		<< 2		// 24.0
		<< 3		// 25.0
		<< 4		// 29.97
		<< 5		// 30.0
		<< 6		// 50.0
		<< 7		// 59.94
		<< 8;		// 60.0

	QList<int> ntscFps, PalFps;
	ntscFps << 1	// 23.976
		<< 4		// 29.97
		<< 7;		// 59.94
	PalFps << 3		// 25.0
		<< 6;		// 50.0

	// Item #2... (push every common fps) 
	while (!commonFps.isEmpty() )
	{
		mpeg_fps = commonFps.takeFirst();

		if ( ntscFps.indexOf( mpeg_fps ) != -1 )
		{
			fpsComboBox->addItem( tr("%1 FPS (NTSC)")
				.arg( Video::strDvdFps(mpeg_fps) ) , mpeg_fps);
		}
		else if ( PalFps.indexOf( mpeg_fps ) != -1 )
		{
			fpsComboBox->addItem( tr("%1 FPS (PAL)")
				.arg( Video::strDvdFps(mpeg_fps) ) , mpeg_fps);
		}
		else
		{
			fpsComboBox->addItem( tr("%1 FPS")
				.arg( Video::strDvdFps(mpeg_fps) ) , mpeg_fps);
		}
	}

	// Select current fps
	int index = fpsComboBox->findData( encSettings->mpeg2.mpeg_fps );

	if ( index == -1 ) // should not happened
	{
		fpsComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.mpeg_fps = fpsComboBox->itemData( 0 ).toInt();
	}
	else
	{
		fpsComboBox->setCurrentIndex( index );
	}
}

void ConfVidMpeg2MEncoderDialog::populateBrAllocation()
{
	br_allocComboBox->clear();

	br_allocComboBox->addItem( tr("CBR - Constant Bit Rate"), CBR );
	br_allocComboBox->addItem( tr("VBR - Variable Bit Rate"), VBR );

	if ( encSettings->mpeg2.cbr )
	{
		br_allocComboBox->setCurrentIndex( 0 ); // CBR
	}
	else
	{
		br_allocComboBox->setCurrentIndex( 1 ); // VBR
	}
}

void ConfVidMpeg2MEncoderDialog::populateAspect()
{
	int mpeg_aspect(0);

	aspectComboBox->clear();

	// Build a list of MPEG fps 
	QList<int> mpegAspect;
	mpegAspect << 1	// 1:1
		<< 2		// 4:3
		<< 3		// 16:9
		<< 4;		// 2.21:1

	aspectComboBox->addItem( tr("Not set"), 0 );

	while (!mpegAspect.isEmpty() )
	{
		mpeg_aspect = mpegAspect.takeFirst();

		if ( (mpeg_aspect == 2) && qAbs(videoSource->display_aspect - 1.33) <= 0.1 )
		{
			aspectComboBox->addItem( tr("%1 (recommended)")
				.arg( Video::strDvdAspect(mpeg_aspect) ) , mpeg_aspect);
		}
		else if ( (mpeg_aspect == 3) && qAbs(videoSource->display_aspect - 1.78) <= 0.1 )
		{
			aspectComboBox->addItem( tr("%1 (recommended)")
				.arg( Video::strDvdAspect(mpeg_aspect) ) , mpeg_aspect);
		}
		else if ( qAbs(videoSource->display_aspect - Video::valDvdAspect(mpeg_aspect)) <= 0.1 )
		{
			aspectComboBox->addItem( tr("%1 (input)")
				.arg( Video::strDvdAspect(mpeg_aspect) ) , mpeg_aspect);
		}
		else
		{
			aspectComboBox->addItem( tr("%1")
				.arg( Video::strDvdAspect(mpeg_aspect) ) , mpeg_aspect);
		}
	}

	// Select current aspect
	int index = aspectComboBox->findData( encSettings->mpeg2.mpeg_aspect );

	if ( index == -1 ) // should not happened
	{
		aspectComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.mpeg_aspect = aspectComboBox->itemData( 0 ).toInt();
	}
	else
	{
		aspectComboBox->setCurrentIndex( index );
	}
}

void ConfVidMpeg2MEncoderDialog::populateTeleSrc()
{
	dgSrcComboBox->clear();

	// Build a list of MPEG fps
	QList<double> listedFps;
	listedFps << 0.0
		<< Video::valDvdFps(1)		// 23.976
		<< Video::valDvdFps(2)		// 24.0
		<< Video::valDvdFps(3)		// 25.0
		<< Video::valDvdFps(4)		// 29.97
		<< Video::valDvdFps(5)		// 30.0
		<< Video::valDvdFps(6)		// 50.0
		<< Video::valDvdFps(7);		// 59.94

	// QMap is ordered and unique key
	QMap<double, QString> mappedFps;

	while (!listedFps.isEmpty() )
	{
		double fps = listedFps.takeFirst();

		if ( fps == 0.0 )
		{
			mappedFps.insert(fps, tr("Not Set"));
		}
		else
		{
			mappedFps.insert(fps, tr("%1 FPS").arg( QString::number(fps, 'f', 3) )); 
		}
	}

	// Ensure input fps, and ordered and not duplicated
	mappedFps.insert(videoSource->fps, tr("%1 FPS (input)").arg( QString::number(videoSource->fps, 'f', 3) ));

	// Now full in the combobox
	QMapIterator<double, QString> fps(mappedFps);
	
	while ( fps.hasNext() )
	{
		fps.next();
		dgSrcComboBox->addItem( fps.value(), fps.key() );
	}
	
	// Select current fps
	int index = dgSrcComboBox->findData( encSettings->mpeg2.tele_src );

	if ( index == -1 ) // should not happened
	{
		dgSrcComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.tele_src = dgSrcComboBox->itemData( 0 ).toDouble();
	}
	else
	{
		dgSrcComboBox->setCurrentIndex( index );
	}
}

void ConfVidMpeg2MEncoderDialog::populateTeleDest()
{
	double fps(0.0);

	dgDestComboBox->clear();

	// Build a list of MPEG fps 
	QList<double> listedFps;
	listedFps << 0
		<< Video::valDvdFps(2)		// 24.0
		<< Video::valDvdFps(3)		// 25.0
		<< Video::valDvdFps(4)		// 29.97
		<< Video::valDvdFps(5)		// 30.0
		<< Video::valDvdFps(6)		// 50.0
		<< Video::valDvdFps(7)		// 59.94
		<< Video::valDvdFps(8);		// 60.0

	while (!listedFps.isEmpty() )
	{
		fps = listedFps.takeFirst();

		if ( fps == 0.0 )
		{
			dgDestComboBox->addItem( tr("Not Set"), fps);
		}
		else
		{
			dgDestComboBox->addItem( tr("%1 FPS")
				.arg( QString::number(fps, 'f', 3) ), fps);
		}
	}

	// Select current fps
	int index = dgDestComboBox->findData( encSettings->mpeg2.tele_dest );

	if ( index == -1 ) // should not happened
	{
		dgDestComboBox->setCurrentIndex( 0 );
		encSettings->mpeg2.tele_dest = dgDestComboBox->itemData( 0 ).toInt();
	}
	else
	{
		dgDestComboBox->setCurrentIndex( index );
	}
}

void ConfVidMpeg2MEncoderDialog::populateRez()
{
	QSize size;
	
	rezComboBox->clear();

	// First must be the recommended size
	if ( tvNtscCheckBox->isChecked() )
	{
		size.setWidth(720);
		size.setHeight(480);
	}
	else if ( tvPalCheckBox->isChecked() )
	{
		size.setWidth(720);
		size.setHeight(576);
	}
	else
	{
		size.setWidth(720);
		size.setHeight(480);
	}

	encSettings->recommendation.mpeg2.resX = size.width();
	encSettings->recommendation.mpeg2.resY = size.height();

	// Item #0
	rezComboBox->addItem( tr("%1x%2 (recommended)")
		.arg(size.width())
		.arg(size.height()) , size);
	
	// Second must be the user defined
	size = QSize(encSettings->user_defined.mpeg2.resX, encSettings->user_defined.mpeg2.resY);

	if (size.width() <= 0)
	{
		size.setWidth(352);
	}

	if (size.height() <= 0)
	{
		size.setHeight(240);
	}

	encSettings->user_defined.mpeg2.resX = size.width();
	encSettings->user_defined.mpeg2.resY = size.height();

	// Item #1
	rezComboBox->addItem( tr("%1x%2 (user defined)")
		.arg(size.width())
		.arg(size.height()) , size);

	// Build a list of most common rez 
	QList<QSize> commonSizes;
	commonSizes << QSize( 480, 480 )
		<< QSize( 480, 576 )
		<< QSize( 720, 480 )
		<< QSize( 720, 576 )
		<< QSize( 1140, 1152 )
		<< QSize( 1920, 1152 );

	QList<QSize> svcdSizes;
	svcdSizes << QSize( 480, 480 )
		<< QSize( 480, 576 );

	QList<QSize> dvdSizes;
	dvdSizes << QSize( 720, 480 )
		<< QSize( 720, 576 );

	// Item #2... (push every common rez) 
	while (!commonSizes.isEmpty() )
	{
		size = commonSizes.takeFirst();

		if ( svcdSizes.indexOf( size ) >= 0 )
		{
			rezComboBox->addItem( tr("%1x%2 (SVCD)")
				.arg(size.width())
				.arg(size.height()) , size);
		}
		else if ( dvdSizes.indexOf( size ) >= 0 )
		{
			rezComboBox->addItem( tr("%1x%2 (DVD)")
				.arg(size.width())
				.arg(size.height()) , size);
		}
		else
		{
			rezComboBox->addItem( tr("%1x%2")
				.arg(size.width())
				.arg(size.height()) , size);
		}
	}

	// Select current rez
	size = QSize(encSettings->mpeg2.resX, encSettings->mpeg2.resY);

	if (size.width() <= 0)
	{
		size.setWidth(352);
	}

	if (size.height() <= 0)
	{
		size.setHeight(240);
	}

	encSettings->mpeg2.resX = size.width();
	encSettings->mpeg2.resY = size.height();

	int index = rezComboBox->findData( size );

	if (index == -1)
	{
		// Missing current rez in list is forced to be the user defined (item #1)
		index = 1;
		rezComboBox->setItemText( index, tr("%1x%2 (user defined)")
			.arg(size.width())
			.arg(size.height()));
		rezComboBox->setItemData( index, size);
	}

	rezComboBox->setCurrentIndex( index );
}

void ConfVidMpeg2MEncoderDialog::bitrateLineEditChanged(const QString &value)
{
	encSettings->mpeg2.bitrate = value.toInt();

	bitrateSlider->setValue( encSettings->mpeg2.bitrate );

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::bitrateSliderChanged(const int value)
{
	encSettings->mpeg2.bitrate = value;

	if ( encSettings->mpeg2.cbr )
	{
		encSettings->mpeg2.minvbr = encSettings->mpeg2.bitrate;
		encSettings->mpeg2.maxvbr = encSettings->mpeg2.bitrate;
	}
	else
	{
		encSettings->mpeg2.minvbr = qMin( encSettings->mpeg2.minvbr, encSettings->mpeg2.bitrate);
		encSettings->mpeg2.maxvbr = qMax( encSettings->mpeg2.maxvbr, encSettings->mpeg2.bitrate);
	}

	bitrateLineEdit->setText( QString::number(value) );

	vbr_nomSlider->setValue( value );

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectPresetDvd()
{
	if ( videoSource->fps == 25.0 || videoSource->fps == 50.0 )
	{
		// TV standard: PAL
		encSettings->mpeg2.dvdPal();
	}
	else
	{
		// TV standard: NTSC
		encSettings->mpeg2.dvdNtsc( videoSource->fps );
	}

	pack_sizeLineEdit->setEnabled(true);
	muxerDefaultPushButton->setEnabled(true);

	populateFps();
	populateRez();
	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectPresetDvdLp()
{
	if ( videoSource->fps == 25.0 || videoSource->fps == 50.0 )
	{
		// TV standard: PAL
		encSettings->mpeg2.dvdPal();
	}
	else
	{
		// TV standard: NTSC
		encSettings->mpeg2.dvdNtsc( videoSource->fps );
	}

	// Long Play bitrate
	encSettings->mpeg2.minvbr = 3000;
	encSettings->mpeg2.bitrate = 4000;
	encSettings->mpeg2.maxvbr = 5000;

	populateFps();
	populateRez();
	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectPresetSvcd()
{
	if ( videoSource->fps == 25.0 )
	{
		// TV standard: PAL
		encSettings->mpeg2.svcdPal();
	}
	else
	{
		// TV standard: NTSC
		encSettings->mpeg2.svcdNtsc( videoSource->fps );
	}

	populateFps();
	populateRez();
	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectRez(const int index)
{
	// Do nothing if empty or reset
	if (index < 0)
	{
		return;
	}

	if (index == 1)
	{
		enterUserDefinedSize();
	}

	QSize size = rezComboBox->itemData(index).toSize();

	encSettings->mpeg2.resX = size.width();
	encSettings->mpeg2.resY = size.height();

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectFps(const int index)
{
	// Do nothing if empty or reset
	if (index < 0)
	{
		return;
	}

	encSettings->mpeg2.mpeg_fps = fpsComboBox->itemData(index).toInt();

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectNtscTv()
{
	encSettings->mpeg2.tvNtsc( videoSource->fps );

	populateFps();
	populateRez();
	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectPalTv()
{
	encSettings->mpeg2.tvPal();

	populateFps();
	populateRez();
	updateControls();
}

void ConfVidMpeg2MEncoderDialog::vbr_nomSliderChanged(const int value)
{
	vbr_nomLineEdit->setText( QString::number(value) );

	bitrateSlider->setValue( value );
}

void ConfVidMpeg2MEncoderDialog::vbr_nomLineEditChanged(const QString &value)
{
	bitrateSlider->setValue( value.toInt() );
}

void ConfVidMpeg2MEncoderDialog::closed_gopCheckBoxClicked()
{
	encSettings->mpeg2.closed_gop = ( closed_gopCheckBox->isChecked() );

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::vbr_minLineEditChanged(const QString &str)
{
	int value = str.toInt();

	if ( value > encSettings->mpeg2.bitrate )
	{
		encSettings->mpeg2.minvbr = encSettings->mpeg2.bitrate;
	}
	else
	{
		encSettings->mpeg2.minvbr = value;
	}

	vbr_minSlider->setValue( encSettings->mpeg2.minvbr );
}

void ConfVidMpeg2MEncoderDialog::vbr_minSliderChanged(const int value)
{
	if ( value > encSettings->mpeg2.bitrate )
	{
		encSettings->mpeg2.minvbr = encSettings->mpeg2.bitrate;
	}
	else
	{
		encSettings->mpeg2.minvbr = value;
	}

	vbr_minLineEdit->setText( QString::number(encSettings->mpeg2.minvbr) );
}

void ConfVidMpeg2MEncoderDialog::vbr_minSliderReleased()
{
	if ( vbr_minSlider->value() > encSettings->mpeg2.bitrate )
	{
		encSettings->mpeg2.minvbr = encSettings->mpeg2.bitrate;
		vbr_minSlider->setValue( encSettings->mpeg2.minvbr );
	}
}

void ConfVidMpeg2MEncoderDialog::vbr_maxLineEditChanged(const QString &str)
{
	int value = str.toInt();

	if ( value < encSettings->mpeg2.bitrate )
	{
		encSettings->mpeg2.maxvbr = encSettings->mpeg2.bitrate;
	}
	else
	{
		encSettings->mpeg2.maxvbr = value;
	}

	vbr_maxSlider->setValue( encSettings->mpeg2.maxvbr );
}

void ConfVidMpeg2MEncoderDialog::vbr_maxSliderChanged(const int value)
{
	if ( vbr_maxSlider->value() < encSettings->mpeg2.bitrate )
	{
		encSettings->mpeg2.maxvbr = encSettings->mpeg2.bitrate;
	}
	else
	{
		encSettings->mpeg2.maxvbr = value;
	}

	vbr_maxLineEdit->setText( QString::number(encSettings->mpeg2.maxvbr) );
}

void ConfVidMpeg2MEncoderDialog::vbr_maxSliderReleased()
{
	if ( vbr_maxSlider->value() < encSettings->mpeg2.bitrate )
	{
		encSettings->mpeg2.maxvbr = encSettings->mpeg2.bitrate;
		vbr_maxSlider->setValue( encSettings->mpeg2.maxvbr );
	}
}

void ConfVidMpeg2MEncoderDialog::gop_sizeLineEditChanged(const QString &str)
{
	encSettings->mpeg2.gop_size = str.toInt();
}

void ConfVidMpeg2MEncoderDialog::selectBrAllocation(const int index)
{
	// Do nothing if combobox is empty or reset
	if (index < 0)
	{
		return;
	}

	if ( br_allocComboBox->itemData( index ).toInt() ==  VBR )
	{
		encSettings->mpeg2.cbr = false;

		encSettings->mpeg2.minvbr = bitrateMin;
		vbr_minSlider->setValue( encSettings->mpeg2.minvbr );
		vbr_minLineEdit->setText( QString::number(encSettings->mpeg2.minvbr) );

		encSettings->mpeg2.maxvbr = bitrateMax;
		vbr_maxSlider->setValue( encSettings->mpeg2.maxvbr );
		vbr_maxLineEdit->setText( QString::number(encSettings->mpeg2.maxvbr) );

		vbr_minLabel->setEnabled(true);
		vbr_minSlider->setEnabled(true);
		vbr_minLineEdit->setEnabled(true);
		vbr_maxLabel->setEnabled(true);
		vbr_maxSlider->setEnabled(true);
		vbr_maxLineEdit->setEnabled(true);		
	}
	else
	{
		encSettings->mpeg2.cbr = true;
		encSettings->mpeg2.minvbr = encSettings->mpeg2.bitrate;
		encSettings->mpeg2.maxvbr = encSettings->mpeg2.bitrate;

		vbr_minSlider->setValue( encSettings->mpeg2.minvbr );
		vbr_minLineEdit->setText( QString::number(encSettings->mpeg2.minvbr) );
		vbr_maxSlider->setValue( encSettings->mpeg2.maxvbr );
		vbr_maxLineEdit->setText( QString::number(encSettings->mpeg2.maxvbr) );

		vbr_minLabel->setEnabled(false);
		vbr_minSlider->setEnabled(false);
		vbr_minLineEdit->setEnabled(false);
		vbr_maxLabel->setEnabled(false);
		vbr_maxSlider->setEnabled(false);
		vbr_maxLineEdit->setEnabled(false);
	}
}


void ConfVidMpeg2MEncoderDialog::selectTelecine()
{
	if ( telecineNoneRadioButton->isChecked() )
	{
		encSettings->mpeg2.film2pal = false;
		encSettings->mpeg2.telecine = false;
		dgSrcComboBox->setCurrentIndex( 0 );
		dgDestComboBox->setCurrentIndex( 0 );
	}
	else if ( telecine32PulldownRadioButton->isChecked() )
	{
		encSettings->mpeg2.film2pal = false;
		encSettings->mpeg2.telecine = true;
		dgSrcComboBox->setCurrentIndex( 0 );
		dgDestComboBox->setCurrentIndex( 0 );
	}
	else if ( telecineFilm2PalRadioButton->isChecked() )
	{
		encSettings->mpeg2.film2pal = true;
		encSettings->mpeg2.telecine = false;
		dgSrcComboBox->setCurrentIndex( 0 );
		dgDestComboBox->setCurrentIndex( 0 );
	}
	else if ( telecineDGPulldownRadioButton->isChecked() )
	{
		encSettings->mpeg2.film2pal = false;
		encSettings->mpeg2.telecine = false;

		int i = dgSrcComboBox->findData( videoSource->fps );

		if (  i > 0 )
		{
			dgSrcComboBox->setCurrentIndex( i );
		}
		else
		{
			dgSrcComboBox->setCurrentIndex( 0 );
		}
	}
	else
	{
		// Should not happen, a bug to pop-up and fix
#ifdef WIN32
#pragma message (Reminder "a bug to report in log - " __FUNCTION__)
#endif
		encSettings->mpeg2.film2pal = false;
		encSettings->mpeg2.telecine = false;
		encSettings->mpeg2.tele_src = 0;
		encSettings->mpeg2.tele_dest = 0;
		dgSrcComboBox->setCurrentIndex( 0 );
		dgDestComboBox->setCurrentIndex( 0 );
	}
}

void ConfVidMpeg2MEncoderDialog::selectDgSrc(const int index)
{
	// Do nothing if empty or reset
	if (index < 0)
	{
		return;
	}

	// Unset DG pulldown
	if ( index == 0 )
	{
		encSettings->mpeg2.tele_src = 0;

		if ( dgDestComboBox->currentIndex() != 0 ) dgDestComboBox->setCurrentIndex( 0 );

		if ( telecineDGPulldownRadioButton->isChecked() )
		{
			telecineNoneRadioButton->setChecked(true);
		}
		return;
	}

	// Process selection
	encSettings->mpeg2.tele_src = dgSrcComboBox->itemData( index ).toDouble();

	// Ensure acceptable tele_dest
	if ( encSettings->mpeg2.tele_src >= encSettings->mpeg2.tele_dest )
	{
		int i = 1;
		
		while (i < dgDestComboBox->count())
		{
			if (dgDestComboBox->itemData( i ).toDouble() > encSettings->mpeg2.tele_src) break;

			++i;
		}

		dgDestComboBox->setCurrentIndex( i );
	}

	if ( index > 0 )
	{
		telecineDGPulldownRadioButton->setChecked(true);
		encSettings->mpeg2.film2pal = false;
		encSettings->mpeg2.telecine = false;
	}
}

void ConfVidMpeg2MEncoderDialog::selectDgDest(const int index)
{
	// Do nothing if empty or reset
	if (index < 0)
	{
		return;
	}

	// Unset DG pulldown
	if ( index == 0 )
	{
		encSettings->mpeg2.tele_dest = 0;

		if ( dgSrcComboBox->currentIndex() != 0 ) dgSrcComboBox->setCurrentIndex( 0 );

		if ( telecineDGPulldownRadioButton->isChecked() )
		{
			telecineNoneRadioButton->setChecked(true);
		}
		return;
	}

	// Process selection
	encSettings->mpeg2.tele_dest = dgDestComboBox->itemData( index ).toDouble();

	// Ensure acceptable tele_src
	if ( encSettings->mpeg2.tele_src >= encSettings->mpeg2.tele_dest
		|| dgSrcComboBox->currentIndex() == 0 )
	{
		int i = dgSrcComboBox->count();

		if (dgSrcComboBox->itemData( i - 1 ).toDouble() >= encSettings->mpeg2.tele_dest)
		{
			while (i > 1 )
			{
				--i;

				if (dgSrcComboBox->itemData( i - 1 ).toDouble() < encSettings->mpeg2.tele_dest) break;
			}
		}

		dgSrcComboBox->setCurrentIndex( i - 1 );
	}

	if ( index > 0 )
	{
		telecineDGPulldownRadioButton->setChecked(true);
		encSettings->mpeg2.film2pal = false;
		encSettings->mpeg2.telecine = false;
	}
}

void ConfVidMpeg2MEncoderDialog::muxerDefaultPushButtonClicked()
{
	encSettings->mpeg2.pack_size = 0;		
	encSettings->mpeg2.muxrate = 0;
	encSettings->mpeg2.tsaf = false;
	encSettings->mpeg2.interleaving2 = false;
	encSettings->mpeg2.vdelay = 0;
	encSettings->mpeg2.adelay = 0;
	encSettings->mpeg2.drop = false;
	encSettings->mpeg2.vbuf_size = 0;
	encSettings->mpeg2.abuf_size = 0;

	encSettings->mpeg2.telecine = false;
	encSettings->mpeg2.film2pal = false;
	encSettings->mpeg2.tele_src = 0;
	encSettings->mpeg2.tele_dest = 0;

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::pictureDefaultPushButtonClicked()
{
	encSettings->mpeg2.mpeg_aspect = 0;

	encSettings->mpeg2.vpswidth = 0;		
	encSettings->mpeg2.vpsheight = 0;

	encSettings->mpeg2.film2pal = false;
	encSettings->mpeg2.telecine = false;
	dgSrcComboBox->setCurrentIndex( 0 );
	dgDestComboBox->setCurrentIndex( 0 );

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::selectFormat(const int index)
{
	// Do nothing if empty or reset
	if (index < 0)
	{
		return;
	}

	encSettings->mpeg2.stream = formatComboBox->itemData(index).toInt();

	updateControls();
	updateMpegMuxerSettings();
}

void ConfVidMpeg2MEncoderDialog::selectAspect(const int index)
{
	// Do nothing if empty or reset
	if (index < 0)
	{
		return;
	}

	encSettings->mpeg2.mpeg_aspect = aspectComboBox->itemData(index).toInt();

	updateControls();
}

void ConfVidMpeg2MEncoderDialog::setToolTip()
{
#ifdef WIN32
	tvGroupBox->setToolTip( tr("Preset a TV standard, see WhatsThis...") );
#else
	tvGroupBox->setToolTip( tr("Preset a TV standard.") );
#endif

	muxerstdGroupBox->setToolTip( simpleToolTip( tr("Standard compliance is revealed by a green light.") ) );

	bitrateGroupBox->setToolTip( boxToolTip("vbitrate", tr("None"), QString("<%1-%2>").arg( bitrateMin ).arg( bitrateMax ),
						tr("Sets the video bitrate in kbit/s for MPEG-1/2 video.")) );
	
	rezComboBox->setToolTip( boxToolTip("vwidth x vheight", "Input Picture Size", "<1-4095> x <1-4095>",
						tr("Set the video width and height when video is MPEG-1/2.")) );
	
	fpsComboBox->setToolTip( boxToolTip("vframerate", tr("None"), "<24000/1001 | 24 | 25 | 30000/1001 | 30 | 50 | 60000/1001 | 60>",
						tr("Sets the framerate for MPEG-1/2 video. This option will be ignored if used with the telecine option.")) );

	presetDvdCheckBox->setToolTip( buttonToolTip("DVD-Video",
						tr("Standard DVD-Video disc, fair quality for bitrate above 6000 kb/s.")) );
	presetDvdLpCheckBox->setToolTip( buttonToolTip("DVD Long-Play",
						tr("Lower the bitrate, lower the quality. May transfer DVD9 to DVD5.")) );
	presetSvcdCheckBox->setToolTip( buttonToolTip("Super-VCD", 
						tr("Super VideoCD, to transfer DVD to CD.")) );

	formatComboBox->setToolTip( boxToolTip("format", "mpeg2", "<mpeg2 | xsvcd | dvd | pes1 | pes2>",
						tr("Stream format. pes1 and pes2 are very broken formats (no pack header and no padding), "
"but VDR uses them; do not choose them unless you know exactly what you are doing.")) );

	vbr_minLabel->setToolTip( boxToolTip("minvbr", tr("None"), QString("<%1-br>").arg( bitrateMin ),
						tr("VBR minimum bitrate allowed in kbit/s.")) );
	vbr_minSlider->setToolTip( vbr_minLabel->toolTip() );
	vbr_minLineEdit->setToolTip( vbr_minLabel->toolTip() );

	vbr_maxLabel->setToolTip( boxToolTip("maxvbr", tr("None"), QString("<br-%1>").arg( bitrateMax ),
						tr("VBR maximum bitrate allowed in kbit/s.")) );
	vbr_maxSlider->setToolTip( vbr_maxLabel->toolTip() );
	vbr_maxLineEdit->setToolTip( vbr_maxLabel->toolTip() );

#ifdef WIN32
	gopGroupBox->setToolTip( tr("GOP settings, see WhatsThis...") );
#else
	gopGroupBox->setToolTip( tr("GOP settings.") );
#endif

	gop_sizeLineEdit->setToolTip( editToolTip("keyint", "<up to 300>") );
	gop_sizeLabel->setToolTip( labelToolTip( tr("15 for NTSC, 12 for PAL"), 
						tr("Ususal values are 15 for NTSC and 12 for PAL. The maximum values are 18 for NTSC and 15 for PAL "
"but you can set up to 300 while 30-40 frames per GOP are already crazy sizes (used by KVCD).")) );

	closed_gopCheckBox->setToolTip( tr("Close all GOPs") );

	pack_sizeLineEdit->setToolTip( editToolTip("size", "<up to 65535>") );
	pack_sizeLabel->setToolTip( labelToolTip( tr("0 for not set"), 
						tr("Pack size in bytes, do not change unless you know exactly what you are doing. "
"Default is 2048, SVCD standard require 2324.")) );

	muxrateLineEdit->setToolTip( editToolTip("muxrate") );
	muxrateLabel->setToolTip( labelToolTip( tr("0 for not set"),
						tr("Nominal muxrate in kbit/s used in the pack headers. Default \"not set\" let the encoder "
"choose the appropriate rate to avoid buffer underflows.")) );

	vdelayLineEdit->setToolTip( editToolTip("vdelay", "<0-32760>") );
	vdelayLabel->setToolTip( labelToolTip(QString::number(0),
						tr("Initial video delay time, in milliseconds, use it if you want to delay video with respect to audio. It doesn't work with :drop.")) );

	adelayLineEdit->setToolTip( editToolTip("adelay", "<0-32760>") );
	adelayLabel->setToolTip( labelToolTip(QString::number(0),
						tr("Initial audio delay time, in milliseconds, use it if you want to delay audio with respect to video.")) );

	vbuf_sizeLineEdit->setToolTip( editToolTip("vbuf_size", "<40-1194>") );
	vbuf_sizeLabel->setToolTip( labelToolTip(tr("Not Set"),
						tr("Sets the size of the video decoder's buffer, expressed in kilo-bytes. Specify it only if the bitrate of the video stream is "
"too high for the chosen format and if you know perfectly well what you are doing. A too high value may lead to an unplayable "
"movie, depending on the player's capabilities. When muxing HDTV video a value of 400 should suffice.")) );

	abuf_sizeLineEdit->setToolTip( editToolTip("abuf_size", "<4-64>") );
	abuf_sizeLabel->setToolTip( labelToolTip(tr("Not Set"),
						tr("Sets the size of the audio decoder's buffer, expressed in kilo-bytes. The same principle as for vbuf_size applies.")) );

	tsafCheckBox->setToolTip( boxToolTip("tsaf", tr("Not Set"), "",
						tr("Sets timestamps on all frames, if possible; recommended when "
"format=dvd. If dvdauthor complains with a message like \"..audio sector out of range...\", you probably did not enable this option.")) );

	interleaving2CheckBox->setToolTip( boxToolTip("interleaving2", tr("Not Set"), "",
						tr("Uses a better algorithm to interleave audio and video packets, "
"based on the principle that the muxer will always try to fill the stream with the largest percentage of free space.")) );

	dropCheckBox->setToolTip( boxToolTip("drop", tr("Not Set"), "",
						tr("When used with vdelay the muxer drops the part of audio that was anticipated.")) );

	aspectGroupBox->setToolTip( boxToolTip("vaspect", "Not Set", "<1 | 4/3 | 16/9 | 221/100>",
						tr("Sets the display aspect ratio.")) );

	panscanGroupBox->setToolTip( boxToolTip("vpswidth x vpsheight", "Not Set", "<1-4095> x <1-4095>",
						tr("Set pan and scan video width and height.")) );

	telecine32PulldownRadioButton->setToolTip( buttonToolTip("telecine", 
						tr("Enables 3:2 pulldown soft telecine mode: The muxer will make the video stream look like "
"it was encoded at 30000/1001 fps. It only works with MPEG-2 video when the output framerate is 24000/1001 fps, "
"convert it with -ofps if necessary. Any other framerate is incompatible with this option.")) );
	telecineFilm2PalRadioButton->setToolTip( buttonToolTip("film2pal", 
						tr("Enables FILM to PAL and NTSC to PAL soft telecine mode: The muxer will make the video "
"stream look like it was encoded at 25 fps. It only works with MPEG-2 video when the output framerate is 24000/1001 "
"fps, convert it with -ofps if necessary. Any other framerate is incompatible with this option.")) );
	telecineDGPulldownRadioButton->setToolTip( buttonToolTip("tele_src , tele_dest", 
						tr("Enables arbitrary telecining using Donand Grafts DGPulldown code. You need to specify "
"the original and the desired framerate; the muxer will make the video stream look like it was encoded at the desired "
"framerate. It only works with MPEG-2 video when the input framerate is smaller than the output framerate and the "
"framerate increase is <= 1.5. EXAMPLE: tele_src=25,tele_dest=30000/1001 (PAL to NTSC telecining).")) );

	dgSrcComboBox->setToolTip( buttonToolTip("tele_src",
						tr("Input framerate must be smaller than the output framerate and the "
"framerate increase must be <= 1.5.")) );

	dgDestComboBox->setToolTip( buttonToolTip("tele_dest",
						tr("Input framerate must be smaller than the output framerate and the "
"framerate increase must be <= 1.5.")) );
}

void ConfVidMpeg2MEncoderDialog::setWhatsThis()
{
#ifdef WIN32
#pragma message (Reminder "why not using QTextDocument with insertHtml(), insertList() and insertTable() - " __FUNCTION__)
#endif

	gopGroupBox->setWhatsThis( tr("I-frames store complete frame info within the frame and is therefore suited for random access. "
"P-frames provide compression using motion vectors relative to the previous frame ( I or P ). B-frames provide maximum compression "
"but requires the previous as well as next frame for computation. Therefore, processing of B-frames require more buffer on the "
"decoded side. A configuration of the Group of Pictures (GOP) should be selected based on these factors. I-frame only sequences "
"gives least compression, but is useful for random access, FF/FR and editability. I and P frame sequences give moderate compression "
"but and a certain degree of random access, FF/FR functionality. I,P & B frame sequences give very high compression but also increases "
"the coding/decoding delay significantly.") );

	tvGroupBox->setWhatsThis( tr("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html>\n"
"<head>\n"
"<meta name=\"qrichtext\" content=\"1\" />\n"
"<style type=\"text/css\">\n"
"p, li {\n"
"	white-space: pre-wrap;\n"
"}\n"
".centercell {\n"
"	text-align: center;\n"
"}\n"
".mybold {\n"
"	font-weight: 600;\n"
"}\n"
"</style>\n"
"</head>\n"
"<body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p>Output movie<span class=\"mybold\"> TV standard</span>.</p>\n"
"<p>\n"
"<table border=\"0.25\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px;\" cellspacing=\"0\" cellpadding=\"0\">\n"
" <tr>\n"
" <td><span class=\"mybold\">NTSC</span> North America </td>\n"
" <td class=\"centercell\">Canada, Mexico, United States </td>\n"
" </tr>\n"
" <tr>\n"
" <td><span class=\"mybold\">NTSC</span> Central America & Caribbean</td>\n"
" <td class=\"centercell\">Antigua and Barbuda, Aruba, Bahamas, Barbados, Belize, Bermuda, British Virgin Islands, Cayman Islands, Costa Rica, Cuba, "
" Dominica, Dominican Republic, El Salvador, Guatemala, Grenada, Haiti, Honduras, Jamaica, Leeward Islands, Montserrat, Netherlands Antilles, Nicaragua, "
" Panama, Puerto Rico (U.S.), Saint Kitts and Nevis, Saint Lucia, Saint Vincent and the Grenadines, Trinidad and Tobago, U.S. Virgin Islands </td>\n"
" </tr>\n"
" <tr>\n"
" <td><span class=\"mybold\">NTSC</span> South America </td>\n"
" <td class=\"centercell\">Bolivia, Colombia, Ecuador, Guyana, Chile, Peru, Suriname, Venezuela </td>\n"
" </tr>\n"
" <tr>\n"
" <td><span class=\"mybold\">NTSC</span> Asia </td>\n"
" <td class=\"centercell\">Japan, South Korea, Republic of China (Taiwan)T, Union of Myanmar (Burma), Philippines </td>\n"
" </tr>\n"
" <tr>\n"
" <td><span class=\"mybold\">PAL</span> (rest of the world)</td>\n"
" <td class=\"centercell\">Afghanistan, Albania, Angola, Ascension, Australia, Austria, Bahrain, Bangladesh, Bosnia and Herzegovina, Botswana, Brunei, Bulgaria,"
" Cameroon, Cape Verde, China, Chile, Christmas Island, Cook Islands, Croatia, Cyprus, Czech Republic, East Timor, Egypt, Eritrea, Ethiopia, Falkland Islands, Fiji, France,"
" Gambia, Georgia, Germany, Ghana, Gibraltar, Greece, Guinea-Bissau, Guinea, Hong Kong, Hungary, Iceland, India, Indonesia, Iran, Iraq, Ireland, Israel, Italy,"
" Jordan, Kenya, Kuwait, Laos, Lebanon, Lesotho, Liberia, Liechtenstein, Lithuania, Macau, Macedonia, Malawi, Malaysia, Maldives, Malta, Montenegro, Morocco,"
" Mozambique, Myanmar, Namibia, Nepal, Netherlands, New Zealand, Nigeria, Norfolk Island, North Korea, Oman, Pakistan, Palestine, Papua New Guinea, Poland, Portugal,"
" Qatar, Romania, Samoa, Saudi Arabia, Serbia, Seychelles, Sierra Leone, Singapore, Slovakia, Slovenia, Solomon Islands, Somalia, South Africa, Sri Lanka,"
" Sudan, Swaziland, Syria, Tanzania, Thailand, Tonga, Tristan da Cunha, Tunisia, Turkey, Uganda, Ukraine, United Arab Emirates, United Kingdom,"
" Vanuatu, Vatican City, Vietnam, Yemen, Zambia, Zanzibar, Zimbabwe </td>\n"
" </tr>\n"
"</table>\n"
"</p>\n"
"<p>NTSC and PAL will be replaced world-wide with DVB-T (Terrestrial Digital Video Broadcast) in the coming years. </p>\n"
"</body>\n"
"</html>") );

	rezComboBox->setWhatsThis( tr("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html>\n"
"<head>\n"
"<meta name=\"qrichtext\" content=\"1\" />\n"
"<style type=\"text/css\">\n"
"p, li {\n"
"	white-space: pre-wrap;\n"
"}\n"
".centercell {\n"
"	text-align: center;\n"
"}\n"
".mybold {\n"
"	font-weight: 600;\n"
"}\n"
"</style>\n"
"</head>\n"
"<body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p>Output movie<span class=\"mybold\"> picture size</span>. Recommended resolutions:</p>\n"
"<p>\n"
"<table border=\"0.25\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px;\" cellspacing=\"0\" cellpadding=\"0\">\n"
" <tr class=\"mybold\">\n"
" <td>Display </td>\n"
" <td class=\"centercell\"> NTSC </td>\n"
" <td class=\"centercell\"> PAL </td>\n"
" </tr>\n"
" <tr>\n"
" <td>Handheld </td>\n"
" <td class=\"centercell\">192x144 </td>\n"
" <td class=\"centercell\">192x144 </td>\n"
" </tr>\n"
" <tr>\n"
""
" <td>Portable </td>\n"
" <td class=\"centercell\">352x240 </td>\n"
" <td class=\"centercell\">352x288 </td>\n"
" </tr>\n"
" <tr>\n"
" <td>Large Handheld </td>\n"
" <td class=\"centercell\">384x288 </td>\n"
" <td class=\"centercell\">384x288 </td>\n"
" </tr>\n"
" <tr>\n"
" <td>Notebook / Desktop </td>\n"
" <td class=\"centercell\">640x360 </td>\n"
" <td class=\"centercell\">640x360 </td>\n"
" </tr>\n"
" <tr>\n"
" <td>Home Theater </td>\n"
" <td class=\"centercell\">720x480 </td>\n"
" <td class=\"centercell\">720x576 </td>\n"
" </tr>\n"
" <tr>\n"
" <td>Maximum MPEG-1 Constrained </td>\n"
" <td class=\"centercell\">768x480 </td>\n"
" <td class=\"centercell\">768x576 </td>\n"
" </tr>\n"
"</table>\n"
"</p>\n"
"<p>Recommended picture height are 480 or 240 for NTSC movies (which are 480-line video sources), and 576 or 288 for PAL movies (which are 576-line video sources). </p>\n"
"</body>\n"
"</html>") );

	fpsComboBox->setWhatsThis( tr("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html>\n"
"<head>\n"
"<meta name=\"qrichtext\" content=\"1\" />\n"
"<style type=\"text/css\">\n"
"p, li {\n"
"	white-space: pre-wrap;\n"
"}\n"
".centercell {\n"
"	text-align: center;\n"
"}\n"
".mybold {\n"
"	font-weight: 600;\n"
"}\n"
"</style>\n"
"</head>\n"
"<body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p>Output movie<span class=\"mybold\"> frame rate</span>. Recommended frame rate (fps, images/second):</p>\n"
"<p>\n"
"<table border=\"0.25\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px;\" cellspacing=\"0\" cellpadding=\"0\">\n"
" <tr class=\"mybold\">\n"
" <td>Display </td>\n"
" <td class=\"centercell\"> NTSC </td>\n"
" <td class=\"centercell\"> PAL </td>\n"
" </tr>\n"
" <tr>\n"
" <td>Handheld </td>\n"
" <td class=\"centercell\">15 FPS</td>\n"
" <td class=\"centercell\">15 FPS</td>\n"
" </tr>\n"
""
" <tr>\n"
" <td>Portable </td>\n"
" <td class=\"centercell\">15 FPS</td>\n"
" <td class=\"centercell\">15 FPS</td>\n"
" </tr>\n"
" <tr>\n"
" <td>Large Handheld </td>\n"
" <td class=\"centercell\">23.976-29.97 FPS</td>\n"
" <td class=\"centercell\">25 FPS</td>\n"
" </tr>\n"
" <tr>\n"
" <td>Notebook / Desktop </td>\n"
" <td class=\"centercell\">23.976-29.97 FPS</td>\n"
" <td class=\"centercell\">25 FPS</td>\n"
" </tr>\n"
" <tr>\n"
" <td>Home Theater </td>\n"
" <td class=\"centercell\">23.976-29.97 FPS</td>\n"
" <td class=\"centercell\">25 FPS</td>\n"
" </tr>\n"
" <tr>\n"
" <td>HDTV </td>\n"
" <td class=\"centercell\">23.976-29.97 FPS</td>\n"
" <td class=\"centercell\">25 FPS</td>\n"
" </tr>\n"
"</table>\n"
"</body>\n"
"</html>") );
}

QString ConfVidMpeg2MEncoderDialog::simpleToolTip( const QString &text )
{
	return tr( "<p>%1</p>" ).arg(Qt::escape(text));
}

QString ConfVidMpeg2MEncoderDialog::labelToolTip( const QString &defaults, const QString &details )
{
	QString strDefault( tr("Default: ") ); 

	return tr(	"<p><b>%1</b> %2</p>"
				"<p></p>"
				"<p>%3</p>" )
				.arg(Qt::escape(strDefault)).arg(Qt::escape(defaults))
				.arg(Qt::escape(details));
}

QString ConfVidMpeg2MEncoderDialog::editToolTip( const QString &name, const QString &range )
{
	return tr("<b>%1</b> %2").arg( Qt::escape(name) ).arg( Qt::escape(range) );
}

QString ConfVidMpeg2MEncoderDialog::boxToolTip( const QString &name, const QString &defaults, const QString &range, const QString &details )
{
	QString strDefault( tr("Default: ") ); 

	return tr(	"<p><b>%1</b> %2</p>"
				"<p></p>"
				"<p><b>%3</b> %4</p>"
				"<p></p>"
				"<p>%5</p>" )
				.arg(Qt::escape(name)).arg(Qt::escape(range))
				.arg(Qt::escape(strDefault)).arg(Qt::escape(defaults))
				.arg(Qt::escape(details));
}

QString ConfVidMpeg2MEncoderDialog::buttonToolTip( const QString &name, const QString &details )
{
	return tr("<b>%1</b>"
				"<p></p>"
				"<p>%2</p>" )
				.arg( Qt::escape(name) ).arg( Qt::escape(details) );
}
