/*****************************************************************************
 *
 *  widgetbridge.h -- Communication bridge within widgets, which can 
 *                    communicate without knowing each others (ensure nice
 *                    independant widget implementation).
 *
 *  Copyright (C) 2010 Starbuck <starbuck@labdv.com>
 *  All rights reserved.
 *
 *  This file is part of the Gui4 module of the DVDx 4 application.
 *
 *  GNU Lesser General Public License Usage
 *  This file may be used under the terms of the GNU Lesser General
 *  Public License version 2.1 as published by the Free Software
 *  Foundation and appearing in the file LICENSE.LGPL included in the
 *  packaging of this file.  Please review the following information to
 *  ensure the GNU Lesser General Public License version 2.1 requirements
 *  will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 *  GNU General Public License Usage
 *  Alternatively, this file may be used under the terms of the GNU
 *  General Public License version 3.0 as published by the Free Software
 *  Foundation and appearing in the file LICENSE.GPL included in the
 *  packaging of this file.  Please review the following information to
 *  ensure the GNU General Public License version 3.0 requirements will be
 *  met: http://www.gnu.org/copyleft/gpl.html.
 *
 *****************************************************************************/

#ifndef WIDGETBRIDGE_H
#define WIDGETBRIDGE_H

#include <QObject>
#include <QStringList>

class WidgetBridge : public QObject
{
	Q_OBJECT

// One unique bridge in the application: a singleton
private:
	WidgetBridge();
	~WidgetBridge();
	static WidgetBridge *singleton;

public:
	static WidgetBridge *getIt();

	void sendData(const QString &message);
	void sendData(const QString &message, const bool data);
	void sendData(const QString &message, const int data);
	void sendData(const QString &message, const double data);
	void sendData(const QString &message, const QString data);
	void sendData(const QString &message, const QStringList data);
	void sendData(const QString &message, void *data);

signals:
	void sendMessage(const QString &message);
	void sendMessage(const QString &message, const bool data);
	void sendMessage(const QString &message, const int data);
	void sendMessage(const QString &message, const double data);
	void sendMessage(const QString &message, const QString data);
	void sendMessage(const QString &message, const QStringList data);
	void sendMessage(const QString &message, void *data);
};

#endif
